@@ -365,13 +365,59 @@ gw_close_all (void)
 static void
 file_dialog_ok (GtkWidget *widget, gboolean *dialog_result)
 {
-	*dialog_result = TRUE;
+	GtkFileSelection *fsel;
+	const gchar *file_name;
+
+	fsel = GTK_FILE_SELECTION (
+		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
+	file_name = gtk_file_selection_get_filename (fsel);
+
+	if (!g_file_test (file_name, G_FILE_TEST_EXISTS)) {
+		const gchar *pattern;
+
+		g_message ("!file exists in file_dialog_ok: %s", file_name);
+
+		if (fsel->selection_entry == NULL)
+			return;
+
+		pattern = 
+			gtk_entry_get_text (GTK_ENTRY (fsel->selection_entry));
+		gtk_file_selection_complete (fsel, pattern);
+
+	} else if (g_file_test (file_name, G_FILE_TEST_IS_DIR)) {
+		/* Change into directory if that's what the user selected */
+		gint name_len;
+		gchar *dir_name;
+
+		name_len = strlen (file_name);
+		if (name_len < 1 || file_name [name_len - 1] != '/') {
+			/* Needed for the file selector */
+			dir_name = g_strconcat (file_name, "/", NULL);
+		} else {
+			dir_name = g_strdup (file_name);
+		}
+
+		gtk_file_selection_set_filename (fsel, dir_name);
+		g_free (dir_name);
+	} else {
+		gtk_widget_hide (GTK_WIDGET (fsel));
+		gtk_main_quit ();
+		*dialog_result = TRUE;
+	}
+}
+
+static void
+file_dialog_cancel (GtkWidget *widget, gpointer user_data)
+{
+	gtk_widget_hide (
+		gtk_widget_get_ancestor (widget, GTK_TYPE_FILE_SELECTION));
 	gtk_main_quit ();
 }
 
 static guint
 file_dialog_delete_event (GtkWidget *widget, GdkEventAny *event)
 {
+	gtk_widget_hide (widget);
 	gtk_main_quit ();
 	return TRUE;
 }
@@ -394,7 +440,7 @@ gw_ask_for_uri (GPdfWindow *gpdf_window, gchar **uri)
 	g_signal_connect (G_OBJECT (fsel->ok_button), "clicked",
 			  G_CALLBACK (file_dialog_ok), &accepted);
 	g_signal_connect (G_OBJECT (fsel->cancel_button), "clicked",
-			  G_CALLBACK (gtk_main_quit), NULL);
+			  G_CALLBACK (file_dialog_cancel), NULL);
 
 	gtk_window_set_position (GTK_WINDOW (fsel), GTK_WIN_POS_MOUSE);
 
@@ -489,6 +535,8 @@ verb_HelpAbout_cb (BonoboUIComponent *uic, gpointer user_data,
 		N_("Derek B. Noonburg, Xpdf author."),
 		N_("Martin Kretzschmar, GNOME port maintainer."),
 		N_("Michael Meeks, GNOME port creator."),
+		/* please localize as R&eacute;mi (R\303\251mi) */
+		N_("Remi Cohen-Scali."),
 		N_("Miguel de Icaza."),
 		N_("Nat Friedman."),
 		N_("Ravi Pratap."),
