@@ -11,8 +11,11 @@
  * Copyright 2002 Martin Kretzschmar
  */
 
+#define GNOME_PRINT_UNSTABLE_API
+
 #include <math.h>
 #include <time.h>
+#include <unistd.h>
 #include "gpdf-control.h"
 #include "gpdf-stock-icons.h"
 #include "gpdf-view.h"
@@ -23,10 +26,16 @@
 #  include "pdf-properties-display.h"
 #  include <bonobo.h>
 #  include <libgnomevfs/gnome-vfs-uri.h>
+#  include <libgnomevfs/gnome-vfs-utils.h>
 #  include "eog-hig-dialog.h"
+#  include <libgnomeprint/gnome-print.h>
+#  include <libgnomeprint/gnome-print-job.h>
+#  include <libgnomeprintui/gnome-print-dialog.h>
+#  include <glade/glade.h>
 #include "gpdf-g-switch.h"
 #include "gpdf-persist-stream.h"
 #include "gpdf-persist-file.h"
+#include "PSOutputDev.h"
 
 #define noPDF_DEBUG
 #ifdef PDF_DEBUG
@@ -74,6 +83,25 @@ enum {
 };
 
 
+typedef struct _GPdfPrintJob GPdfPrintJob;
+
+struct _GPdfPrintJob {
+	PDFDoc *pdf_doc;
+	GnomePrintJob *gnome_print_job;
+	gint page_start;
+	gint page_end;
+	gint page_next;
+
+	guint idle_id;
+	gchar *file_name;
+	PSOutputDev *ps_output_dev;
+
+	GtkWidget *progress_dialog;
+	GtkProgressBar *progress_bar;
+	GtkLabel *progress_label;
+	GtkWidget *progress_button;
+};
+
 static void
 ui_component_set_sensitive (BonoboUIComponent *ui,
 			    const char *path,
@@ -97,6 +125,7 @@ gpdf_control_enable_ui (GPdfControl *control, gboolean sensitive)
 
 	bonobo_ui_component_freeze (uic, NULL);
 
+	ui_component_set_sensitive (uic, "/commands/FilePrint", sensitive);
 	ui_component_set_sensitive (uic, "/commands/FileProperties", sensitive);
 
 	gpdf_view = GPDF_VIEW (control->priv->gpdf_view);
@@ -123,6 +152,329 @@ gpdf_control_enable_ui (GPdfControl *control, gboolean sensitive)
 	bonobo_ui_component_thaw (uic, NULL);	
 }
 
+static void
+gpdf_print_job_prepare (GPdfPrintJob *pj)
+{
+	GnomePrintConfig *config;
+	gdouble width, height;
+
+	config = gnome_print_job_get_config (pj->gnome_print_job);
+
+	/* width and height are returned in PS points :-) */
+	gnome_print_config_get_page_size (config, &width, &height);
+
+	globalParams->setPSPaperWidth ((int)width);
+	globalParams->setPSPaperHeight ((int)height);
+	globalParams->setPSDuplex (gTrue);
+}
+
+static gboolean
+idle_print_handler (GPdfPrintJob *pj)
+{
+	if (pj->ps_output_dev == NULL) {
+		pj->ps_output_dev = new PSOutputDev (pj->file_name,
+						     pj->pdf_doc->getXRef (),
+						     pj->pdf_doc->getCatalog (),
+						     pj->page_start,
+						     pj->page_end,
+						     psModePS);
+
+		gtk_widget_set_sensitive (pj->progress_button, TRUE);
+		pj->page_next = pj->page_start;
+		return TRUE;
+	}
+
+	if (pj->page_next <= pj->page_end) {
+		gchar *label;
+
+		label = g_strdup_printf (_("Printing: Page %d."), pj->page_next);
+		gtk_label_set_text (pj->progress_label, label);
+		g_free (label);
+
+		gtk_progress_bar_set_fraction (
+			pj->progress_bar,
+			((double)pj->page_next - pj->page_start + 1) /
+			((double)pj->page_end - pj->page_start + 1));
+
+		pj->pdf_doc->displayPage (pj->ps_output_dev,
+					  pj->page_next,
+					  72, 0, gFalse);
+		pj->page_next++;
+		return TRUE;
+	} else {
+		pj->idle_id = 0;
+		delete pj->ps_output_dev;
+		pj->ps_output_dev = NULL;
+		gtk_dialog_response (GTK_DIALOG (pj->progress_dialog),
+				     GTK_RESPONSE_OK);
+		return FALSE; /* Remove this idle source */
+	}
+}
+
+void
+gpdf_control_print_job_print (GPdfControl *control, GPdfPrintJob *pj)
+{
+	gint fd;
+	GladeXML *progress_xml;
+	GSource *idle_source;
+	gint response;
+
+	bonobo_object_ref (BONOBO_OBJECT (control)); /* Keep the pdf-doc alive */
+	fd = g_file_open_tmp ("gpdf-print.ps.XXXXXX", &pj->file_name, NULL);
+	if (fd <= -1)
+		return;
+
+	gnome_print_job_set_file (pj->gnome_print_job, pj->file_name);
+
+	gpdf_print_job_prepare (pj);
+	
+	progress_xml = glade_xml_new (
+		DATADIR "/gpdf/glade/gpdf-print-progress-dialog.glade",
+		NULL, NULL);
+	glade_xml_signal_autoconnect (progress_xml);
+	pj->progress_dialog = 
+		glade_xml_get_widget (progress_xml, "print-progress-dialog");
+	pj->progress_bar =
+		GTK_PROGRESS_BAR (glade_xml_get_widget (progress_xml,
+							"progressbar"));
+	pj->progress_label =
+		GTK_LABEL (glade_xml_get_widget (progress_xml,
+						 "status-label"));
+
+	pj->progress_button = glade_xml_get_widget (progress_xml,
+						    "cancel-button");
+	g_object_unref (G_OBJECT (progress_xml));
+
+	bonobo_control_set_transient_for (BONOBO_CONTROL (control),
+					  GTK_WINDOW (pj->progress_dialog),
+					  NULL);
+
+	idle_source = g_idle_source_new ();
+	g_source_set_priority (idle_source, G_PRIORITY_DEFAULT_IDLE);
+	g_source_set_closure (idle_source,
+			      g_cclosure_new (G_CALLBACK (idle_print_handler),
+					      pj, NULL));
+	pj->idle_id = g_source_attach (idle_source, NULL);
+	g_source_unref (idle_source);
+	
+	response = gtk_dialog_run (GTK_DIALOG (pj->progress_dialog));
+
+	gtk_widget_hide (pj->progress_dialog);
+	gtk_widget_destroy (pj->progress_dialog);
+
+	if (response == GTK_RESPONSE_OK) {
+		gnome_print_job_print (pj->gnome_print_job);
+	} else {
+		g_assert (pj->idle_id);
+
+		g_source_remove (pj->idle_id);
+	}
+
+	close (fd);
+	unlink (pj->file_name);
+	g_free (pj->file_name);
+	g_object_unref (pj->gnome_print_job);
+	bonobo_object_unref (BONOBO_CONTROL (control));
+}
+
+static gboolean
+using_postscript_printer (GnomePrintConfig *config)
+{
+	const guchar *driver;
+	const guchar *transport;
+
+	driver = gnome_print_config_get (
+		config, (const guchar *)"Settings.Engine.Backend.Driver");
+	
+	transport = gnome_print_config_get (
+		config, (const guchar *)"Settings.Transport.Backend");
+	
+	if (driver) {
+		if (!strcmp ((const gchar *)driver, "gnome-print-ps"))
+			return TRUE;
+		else 
+			return FALSE;
+	} else 	if (transport) {
+		if (!strcmp ((const gchar *)transport, "CUPS"))
+			return TRUE;
+	}
+	
+	return FALSE;
+}
+
+static void
+gpdf_control_no_preview_alert (GPdfControl *control)
+{
+	GtkWidget *dlg;
+
+	dlg = eog_hig_dialog_new (
+		GTK_STOCK_DIALOG_ERROR,
+		_("Print preview is not available in this program."),
+		NULL, FALSE);
+	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
+	g_signal_connect_swapped (G_OBJECT (dlg), "response",
+				  G_CALLBACK (gtk_widget_destroy), dlg);
+	bonobo_control_set_transient_for (BONOBO_CONTROL (control),
+					  GTK_WINDOW (dlg), NULL);
+	gtk_widget_show (dlg);
+}
+
+static void
+gpdf_control_no_postscript_printer_alert (GPdfControl *control,
+					  GnomePrintConfig *config)
+{
+	GtkWidget *dlg;
+	const guchar *driver;
+	gchar *secondary;
+
+	driver = gnome_print_config_get (
+		config, (const guchar *)"Settings.Engine.Backend.Driver");
+
+	if (driver)
+		secondary = g_strdup_printf (
+			_("You were trying to print to a printer using the \"%s\" driver. This program requires a PostScript printer driver."),
+			(const gchar *)driver);
+	else
+		secondary = g_strdup (_("You were trying to print to a printer that does not use PostScript. This program requires a PostScript printer driver."));
+
+	dlg = eog_hig_dialog_new (
+		GTK_STOCK_DIALOG_ERROR,
+		_("Printing is not supported on this printer."),
+		secondary, FALSE);
+	g_free (secondary);
+	gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
+	g_signal_connect_swapped (G_OBJECT (dlg), "response",
+				  G_CALLBACK (gtk_widget_destroy), dlg);
+	bonobo_control_set_transient_for (BONOBO_CONTROL (control),
+					  GTK_WINDOW (dlg), NULL);
+	gtk_widget_show (dlg);
+}
+
+static void
+gpdf_control_get_range_page (GPdfControl *control, GnomePrintDialog *dialog,
+			     gint *start, gint *end)
+{
+	switch (gnome_print_dialog_get_range (dialog)) {
+	case GNOME_PRINT_RANGE_ALL:
+		*start = 1;
+		*end = control->priv->pdf_doc->getNumPages ();
+		break;
+	case GNOME_PRINT_RANGE_RANGE:
+		gnome_print_dialog_get_range_page (dialog, start, end);
+		break;
+	case GNOME_PRINT_RANGE_CURRENT:
+		g_warning ("Cannot happen: GNOME_PRINT_RANGE_CURRENT");
+		break;
+	default:
+		g_warning ("Unknown page range type");
+		break;
+	}
+}
+
+static GtkWidget *
+gpdf_control_print_dialog_new (GPdfControl *control, GnomePrintJob *job)
+{
+	GtkWidget *dialog;
+	gchar *pages_label;
+
+	dialog = gnome_print_dialog_new (job,
+					 (const guchar*) _("Print"),
+					 GNOME_PRINT_DIALOG_RANGE);
+
+	pages_label = g_strconcat (_("_Pages"), " ", NULL);
+	gnome_print_dialog_construct_range_page (
+		GNOME_PRINT_DIALOG (dialog),
+		/* (GNOME_PRINT_RANGE_CURRENT | would be the default -> very bad */
+		(GNOME_PRINT_RANGE_ALL | GNOME_PRINT_RANGE_RANGE),
+		1, control->priv->pdf_doc->getNumPages (),
+		NULL, (const guchar *)pages_label);
+	g_free (pages_label);
+
+	bonobo_control_set_transient_for (BONOBO_CONTROL (control),
+					  GTK_WINDOW (dialog),
+					  NULL);
+
+	return dialog;
+}
+
+static void /* FIXME */
+print_to_file_workaround (GnomePrintJob *gpj)
+{
+	const guchar *backend_filename;
+
+	backend_filename = gnome_print_config_get (
+		gnome_print_job_get_config (gpj),
+		(const guchar*)"Settings.Transport.Backend.FileName");
+
+	if (backend_filename)
+		gnome_print_config_set (
+			gnome_print_job_get_config (gpj),
+			(const guchar*)"Settings.Output.Job.FileName",
+			backend_filename);
+}
+
+void
+gpdf_control_print (GPdfControl *control)
+{
+	GPdfPrintJob *pj;
+	GnomePrintConfig *config;
+	GnomePrintJob *job;
+	GtkWidget *print_dialog;
+	gint response;
+	GdkWindow *view_window;
+	GdkDisplay *display;
+	GdkCursor *busy_cursor;
+
+	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (control));
+
+	gpdf_control_enable_ui (control, FALSE);
+	config = gnome_print_config_default ();
+	job = gnome_print_job_new (config);
+
+	print_dialog = gpdf_control_print_dialog_new (control, job);
+	response = gtk_dialog_run (GTK_DIALOG (print_dialog));
+	gtk_widget_hide (print_dialog);
+
+	if (response == GNOME_PRINT_DIALOG_RESPONSE_PREVIEW) {
+		gpdf_control_no_preview_alert (control);
+		goto exit;
+	}
+
+	if (response != GNOME_PRINT_DIALOG_RESPONSE_PRINT)
+		goto exit;
+
+	if (!using_postscript_printer (config)) {
+		gpdf_control_no_postscript_printer_alert (control, config);
+		goto exit;
+	}
+
+	pj = g_new0 (GPdfPrintJob, 1);
+
+	gpdf_control_get_range_page (
+		control, GNOME_PRINT_DIALOG (print_dialog),
+		&pj->page_start, &pj->page_end);
+
+	gtk_widget_destroy (print_dialog);
+
+	print_to_file_workaround (job);
+
+	pj->pdf_doc = control->priv->pdf_doc;
+	pj->gnome_print_job = job;
+
+	view_window = GTK_WIDGET (control->priv->gpdf_view)->window;
+	display = gtk_widget_get_display (GTK_WIDGET (control->priv->gpdf_view));
+	busy_cursor = gdk_cursor_new_for_display (display, GDK_WATCH);
+	gdk_window_set_cursor (view_window, busy_cursor);
+	gdk_cursor_unref (busy_cursor);
+	gdk_flush ();
+	gpdf_control_print_job_print (control, pj);
+	g_free (pj);
+	if (GDK_IS_WINDOW (view_window))
+		gdk_window_set_cursor (view_window, NULL);
+exit:
+	gpdf_control_enable_ui (control, TRUE);
+}
+
 static void
 gpdf_control_properties_dialog_response_cb (GtkDialog *dialog, gint response_id,
 					    gpointer data)
@@ -142,10 +494,14 @@ verb_FileProperties_cb (BonoboUIComponent *uic, gpointer user_data,
 
 	dialog = gtk_dialog_new_with_buttons (_("PDF Properties"),
 					      NULL,
-					      (GtkDialogFlags)0,
+					      GTK_DIALOG_NO_SEPARATOR,
 					      GTK_STOCK_CLOSE,
 					      GTK_RESPONSE_CLOSE,
 					      NULL);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
 	g_signal_connect (
 		GTK_WIDGET (dialog),
 		"response",
@@ -164,6 +520,15 @@ verb_FileProperties_cb (BonoboUIComponent *uic, gpointer user_data,
 	gtk_widget_show (dialog);
 }
 
+static void
+verb_FilePrint_cb (BonoboUIComponent *uic, gpointer user_data,
+		   const char *cname)
+{
+	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (user_data));
+
+	gpdf_control_print (GPDF_CONTROL (user_data));
+}
+
 
 static void
 verb_GoHistoryPrev_cb (BonoboUIComponent *uic, gpointer user_data,
@@ -495,10 +860,11 @@ gc_set_zoom_items_visibility (GPdfControl *control, gboolean visible)
 
 
 static BonoboUIVerb gc_verbs [] = {
+	BONOBO_UI_VERB ("FilePrint",      verb_FilePrint_cb),
 	BONOBO_UI_VERB ("FileProperties", verb_FileProperties_cb),
 
-	BONOBO_UI_VERB ("GoHistoryPrev",     verb_GoHistoryPrev_cb),
-	BONOBO_UI_VERB ("GoHistoryNext",     verb_GoHistoryNext_cb),
+	BONOBO_UI_VERB ("GoHistoryPrev",  verb_GoHistoryPrev_cb),
+	BONOBO_UI_VERB ("GoHistoryNext",  verb_GoHistoryNext_cb),
 
 	BONOBO_UI_VERB ("GoPagePrev",     verb_GoPagePrev_cb),
 	BONOBO_UI_VERB ("GoPageNext",     verb_GoPageNext_cb),
@@ -719,11 +1085,20 @@ persist_file_loading_finished_cb (GPdfPersistFile *persist_file,
 	vfs_uri = uri ? gnome_vfs_uri_new (uri) : NULL;
 
 	if (vfs_uri) {
-		control->priv->title = gnome_vfs_uri_is_local (vfs_uri) ?
-			g_strdup (gnome_vfs_uri_get_path (vfs_uri)) :
-			gnome_vfs_uri_to_string (vfs_uri,
-						 GNOME_VFS_URI_HIDE_NONE);
-                gnome_vfs_uri_unref (vfs_uri);
+		/* Fix for bug #123638: unescape title */
+		const char *title =
+			gnome_vfs_uri_to_string
+			    (vfs_uri,
+			     (GnomeVFSURIHideOptions)
+			     (GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD |
+			      GNOME_VFS_URI_HIDE_HOST_PORT |
+			      GNOME_VFS_URI_HIDE_HOST_NAME |
+			      GNOME_VFS_URI_HIDE_PASSWORD |
+			      GNOME_VFS_URI_HIDE_USER_NAME));
+		control->priv->title =
+			gnome_vfs_unescape_string_for_display (title);
+		g_free ((gpointer)title);
+		gnome_vfs_uri_unref (vfs_uri);
 	} else {
 		control->priv->title = g_strdup (_("PDF Document"));
 	}
@@ -806,9 +1181,17 @@ gpdf_control_set_property (GObject *object, guint param_id,
 		
 		gpdf_control->priv->persist_stream = pstream;
 
+/*
+ * Disable PersistStream because with it we can't close the control
+ * while still printing.  Otherwise we'll get errors.  The
+ * BonoboStream will be closed while we're still trying to read from
+ * it.  (At least with Nautilus.)
+ */
+#if 0
 		bonobo_object_ref (BONOBO_OBJECT (pstream));
 		bonobo_object_add_interface (BONOBO_OBJECT (gpdf_control),
 					     BONOBO_OBJECT (pstream));
+#endif
 		break;
 	}
 	case PROP_PERSIST_FILE: {
