@@ -29,6 +29,7 @@
 #  include <libgnomevfs/gnome-vfs.h>
 #include "gpdf-g-switch.h"
 #include "Object.h"
+#include "ErrorCodes.h"
 #include "PDFDoc.h"
 #include "GnomeVFSStream.h"
 
@@ -50,7 +51,8 @@ BONOBO_CLASS_BOILERPLATE_FULL (GPdfPersistFile, gpdf_persist_file,
 			       BonoboPersist, PARENT_TYPE);
 
 enum {
-	SET_PDF_SIGNAL,
+	LOADING_FINISHED_SIGNAL,
+	LOADING_FAILED_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -100,6 +102,7 @@ impl_bonobo_persist_file_load (PortableServer_Servant servant,
 	GnomeVFSResult vfs_result;
 	Object obj;
 	BaseStream *pdf_stream;
+	const gchar *error_msg;
 
 	bonobo_return_if_fail (GPDF_IS_NON_NULL_PERSIST_FILE (pf), ev);
 
@@ -114,26 +117,52 @@ impl_bonobo_persist_file_load (PortableServer_Servant servant,
 		&priv->vfs_handle, uri,
 		(GnomeVFSOpenMode) (GNOME_VFS_OPEN_READ |
 				    GNOME_VFS_OPEN_RANDOM));
-	/* FIXME handle result */
+	if (vfs_result != GNOME_VFS_OK) {
+		g_signal_emit (G_OBJECT (pf),
+			       gpdf_persist_file_signals [LOADING_FAILED_SIGNAL],
+			       0, gnome_vfs_result_to_string (vfs_result));
+		return;
+	}
 
 	obj.initNull ();
 	pdf_stream = new GnomeVFSStream (priv->vfs_handle, 0, gFalse, 0, &obj);
+	/* FIXME if (pdf_stream == NULL) ? */
 	priv->pdf_doc = new PDFDoc (pdf_stream);
 
-	if (!priv->pdf_doc->isOk ()) {
-		g_warning ("Duff pdf data");
-		goto exit_bad_file;
+	/* FIXME if (pdf_doc == NULL) ? */
+	if (priv->pdf_doc->isOk ()) {
+		g_signal_emit (G_OBJECT (pf),
+			       gpdf_persist_file_signals [LOADING_FINISHED_SIGNAL],
+			       0);
+		return;
 	}
 
-	goto exit_clean;
+	switch (priv->pdf_doc->getErrorCode ()) {
+	case errBadCatalog:
+		/* translators: page catalog is a part of the PDF file */
+		/* The last period (.) is missing on purpose */
+		error_msg = _("The PDF file is damaged. Its page catalog could not be read");
+		break;
+	case errDamaged:
+		error_msg = _("The PDF file is damaged or it is not a PDF file. It could not be repaired");
+		break;
+	case errEncrypted:
+		error_msg = _("The PDF file is encrypted and requires a password. This program does not support password-protected files");
+		break;
+	case errNone:
+	case errOpenFile:
+	case errHighlightFile:
+	default:
+		error_msg = NULL;
+		g_assert_not_reached ();
+		break;
+	}			
 
-exit_bad_file:
-	gpdf_persist_file_unload (pf);
-
-exit_clean:
 	g_signal_emit (G_OBJECT (pf),
-		       gpdf_persist_file_signals [SET_PDF_SIGNAL],
-		       0);	
+		       gpdf_persist_file_signals [LOADING_FAILED_SIGNAL],
+		       0, error_msg);
+	
+	gpdf_persist_file_unload (pf);
 }
 
 static void
@@ -165,13 +194,13 @@ impl_bonobo_persist_file_getCurrentFile (PortableServer_Servant servant,
 		/* otherwise, raise a `NoCurrentName' exception */
 		Bonobo_PersistFile_NoCurrentName *exception;
 		exception = Bonobo_PersistFile_NoCurrentName__alloc ();
-		
+
 		exception->extension = CORBA_string_dup ("");
 		CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
 				     ex_Bonobo_PersistFile_NoCurrentName,
 				     exception);
 		return NULL;
-	}	
+	}
 }
 
 const char *
@@ -189,7 +218,7 @@ gpdf_persist_file_destroy (BonoboObject *object)
 
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GPDF_IS_PERSIST_FILE (object));
-	
+
 	BONOBO_CALL_PARENT (BONOBO_OBJECT_CLASS, destroy, (object));
 }
 
@@ -207,38 +236,49 @@ gpdf_persist_file_finalize (GObject *object)
 static void
 gpdf_persist_file_class_init (GPdfPersistFileClass *klass)
 {
-  	GObjectClass *object_class = G_OBJECT_CLASS (klass);
- 	BonoboObjectClass *bonobo_object_class = BONOBO_OBJECT_CLASS (klass);
- 	BonoboPersistClass *persist_class = BONOBO_PERSIST_CLASS (klass);
- 	POA_Bonobo_PersistFile__epv *epv = &klass->epv;
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	BonoboObjectClass *bonobo_object_class = BONOBO_OBJECT_CLASS (klass);
+	BonoboPersistClass *persist_class = BONOBO_PERSIST_CLASS (klass);
+	POA_Bonobo_PersistFile__epv *epv = &klass->epv;
 
-  	object_class->finalize = gpdf_persist_file_finalize;
- 	bonobo_object_class->destroy = gpdf_persist_file_destroy;
+	object_class->finalize = gpdf_persist_file_finalize;
+	bonobo_object_class->destroy = gpdf_persist_file_destroy;
 	persist_class->get_content_types = impl_get_content_types;
 
 	epv->load = impl_bonobo_persist_file_load;
 	epv->save = impl_bonobo_persist_file_save;
 	epv->getCurrentFile = impl_bonobo_persist_file_getCurrentFile;
 
-	gpdf_persist_file_signals [SET_PDF_SIGNAL] =
-		g_signal_new ("set_pdf",
+	gpdf_persist_file_signals [LOADING_FINISHED_SIGNAL] =
+		g_signal_new ("loading_finished",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GPdfPersistFileClass,
-					       set_pdf),
+					       loading_finished),
 			      NULL, NULL,
 			      gpdf_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+
+	gpdf_persist_file_signals [LOADING_FAILED_SIGNAL] =
+		g_signal_new ("loading_failed",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GPdfPersistFileClass,
+					       loading_failed),
+			      NULL, NULL,
+			      gpdf_marshal_VOID__STRING,
+			      G_TYPE_NONE, 1,
+			      G_TYPE_STRING);
 }
 
 static void
 gpdf_persist_file_instance_init (GPdfPersistFile *persist_file)
 {
- 	persist_file->priv = g_new0 (GPdfPersistFilePrivate, 1);
+	persist_file->priv = g_new0 (GPdfPersistFilePrivate, 1);
 }
 
 GPdfPersistFile *
-gpdf_persist_file_construct (GPdfPersistFile *persist_file, 
+gpdf_persist_file_construct (GPdfPersistFile *persist_file,
 			     const gchar *iid)
 {
 	return GPDF_PERSIST_FILE (
@@ -250,7 +290,7 @@ gpdf_persist_file_new (const gchar *iid)
 {
 	GPdfPersistFile *persist_file;
 
-	persist_file = 
+	persist_file =
 	     GPDF_PERSIST_FILE (g_object_new (GPDF_TYPE_PERSIST_FILE, NULL));
 
 	return gpdf_persist_file_construct (persist_file, iid);
