@@ -38,10 +38,16 @@ G_BEGIN_DECLS
 #define GPDF_IS_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GPDF_TYPE_VIEW))
 #define GPDF_IS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GPDF_TYPE_VIEW))
 
+typedef struct _GPdfViewHistory GPdfViewHistory;
 typedef struct _GPdfView        GPdfView;
 typedef struct _GPdfViewClass   GPdfViewClass;
 typedef struct _GPdfViewPrivate GPdfViewPrivate;
 
+struct _GPdfViewHistory {
+	double zoom;
+	int page; 
+}; 
+
 struct _GPdfView {
 	GnomeCanvas parent_object;
 
@@ -58,26 +64,35 @@ struct _GPdfViewClass {
 	void (*quit_requested)  (GPdfView *gpdf_view);
 };
 
-GType      gpdf_view_get_type       (void);
-GtkWidget *gpdf_view_new            (void);
+GType      gpdf_view_get_type         (void);
+GtkWidget *gpdf_view_new              (void);
 
-void       gpdf_view_set_pdf_doc    (GPdfView *gpdf_view, PDFDoc *pdf_doc);
+void       gpdf_view_set_pdf_doc      (GPdfView *gpdf_view, PDFDoc *pdf_doc);
 
-void       gpdf_view_page_prev      (GPdfView *gpdf_view);
-void       gpdf_view_page_next      (GPdfView *gpdf_view);
-void       gpdf_view_page_first     (GPdfView *gpdf_view);
-void       gpdf_view_page_last      (GPdfView *gpdf_view);
-void       gpdf_view_goto_page      (GPdfView *gpdf_view, gint page);
+void       gpdf_view_scroll_to_top    (GPdfView *gpdf_view);
+void       gpdf_view_scroll_to_bottom (GPdfView *gpdf_view);
 
-gboolean   gpdf_view_is_first_page  (GPdfView *gpdf_view);
-gboolean   gpdf_view_is_last_page   (GPdfView *gpdf_view);
+void       gpdf_view_page_prev        (GPdfView *gpdf_view);
+void       gpdf_view_page_next        (GPdfView *gpdf_view);
+void       gpdf_view_page_first       (GPdfView *gpdf_view);
+void       gpdf_view_page_last        (GPdfView *gpdf_view);
+void       gpdf_view_goto_page        (GPdfView *gpdf_view, gint page);
 
-void       gpdf_view_zoom           (GPdfView *gpdf_view, gdouble factor, gboolean relative);
-void       gpdf_view_zoom_default   (GPdfView *gpdf_view);
-void       gpdf_view_zoom_in        (GPdfView *gpdf_view);
-void       gpdf_view_zoom_out       (GPdfView *gpdf_view);
-void       gpdf_view_zoom_fit_width (GPdfView *gpdf_view, gint width);
-void       gpdf_view_zoom_fit       (GPdfView *gpdf_view, gint width, gint height);
+gboolean   gpdf_view_is_first_page    (GPdfView *gpdf_view);
+gboolean   gpdf_view_is_last_page     (GPdfView *gpdf_view);
+
+void       gpdf_view_back_history     (GPdfView *gpdf_view);
+void       gpdf_view_forward_history  (GPdfView *gpdf_view);
+
+gboolean   gpdf_view_is_first_history (GPdfView *gpdf_view);
+gboolean   gpdf_view_is_last_history  (GPdfView *gpdf_view);
+
+void       gpdf_view_zoom             (GPdfView *gpdf_view, gdouble factor, gboolean relative);
+void       gpdf_view_zoom_default     (GPdfView *gpdf_view);
+void       gpdf_view_zoom_in          (GPdfView *gpdf_view);
+void       gpdf_view_zoom_out         (GPdfView *gpdf_view);
+void       gpdf_view_zoom_fit_width   (GPdfView *gpdf_view, gint width);
+void       gpdf_view_zoom_fit         (GPdfView *gpdf_view, gint width, gint height);
 
 G_END_DECLS
 
